@@ -12,14 +12,7 @@ my $loglevel=3;
 my $logging =0;
 my $logfile = "/tmp/".getpwuid($>)."-audit.log";
 
-$VERSION = '1.11';
-
-sub import {
-    my $pkg = shift;
-    for (@_) {
-         eval "use $pkg"."::$_"; die $@ if $@;
-    }
-}
+$VERSION = '1.2';
 
 sub _log {
     my ($priority, $what) = @_; 
@@ -27,14 +20,7 @@ sub _log {
 }
 
 sub new { 
-    my $class = shift;
-    my %opts = @_;
-    my $self = bless({ 
-        %opts, 
-        obj => Mail::Internet->new(
-                    exists $opts{data}? $opts{data} : \*STDIN
-                ) 
-    }, $class) ;
+    my $self = bless({ obj => Mail::Internet->new(\*STDIN), @_ }, shift) 
     if (exists $self->{loglevel}) { 
         $logging =1;
         $loglevel = $self->{loglevel};
@@ -46,16 +32,16 @@ sub new {
     if ($logging) {
         open LOG, ">>$logfile" or die $!;
         _log(1,"Logging started at ".scalar localtime);
-        _log(2,"Incoming mail from ".$self->from);
-        _log(2,"To: ".$self->to);
-        _log(2,"Subject: ".$self->subject);
+        _log(2,"Incoming mail from ".$mail->from);
+        _log(2,"To: ".$mail->to);
+        _log(2,"Subject: ".$mail->subject);
     }
     return $self;
 }
 
 sub accept {
 	my $self = shift;
-	my $file = shift || $ENV{MAIL} || "/var/spool/mail/".getpwuid($>);
+	my $file = shift || "/var/spool/mail/".getpwuid($>);
     _log(1,"Accepting");
 	return $self->{accept}->() if exists $self->{accept};
     _log(2,"Accepting to $file");
@@ -71,7 +57,7 @@ sub accept {
 		unless ($self->{obj}->head->get("Lines")) {
 			my $body = $self->{obj}->body;
 			my $num_lines = @$body;
-			$self->{obj}->head->add("Lines", $num_lines);
+			$self->{obj}->head->add("Lines", $num_lines)
             _log(4,"Adding Lines: $num_lines header");
 		}
 		unless (open TMP, ">$tmp_path") {
@@ -93,70 +79,107 @@ sub accept {
         }
 		flock(FH, LOCK_EX) 
             or _log(1,"Couldn't get exclusive lock on $file");
-        seek FH, 0, 2;
-		if ((${$self->{obj}->body}[0] !~ /^From\s/) && (exists $ENV{UFLINE})) {
-		    _log(3,"Looks qmail, but preline not run, prepending UFLINE, RPLINE, DTLINE");
-			print FH $ENV{UFLINE};
-			print FH $ENV{RPLINE};
-			print FH $ENV{DTLINE};
-		}
 		print FH $self->{obj}->as_mbox_string;
-		flock(FH, LOCK_UN)
+		flock(FH, LOCK_UN);
             or _log(1,"Couldn't unlock on $file");
 		close FH;
 	}
-    if (!$self->{noexit}) {
-        _log(2,"Exiting with status ".DELIVERED);
+    _log(2,"Exiting with status ".DELIVERED);
 	exit DELIVERED;
-    }
 }
 
 sub reject {
 	my $self=shift;
 	return $self->{reject}->() if exists $self->{reject};
-        _log(1, "Rejecting with reason @_");
+    _log(1, "Rejecting with reason @_");
 	print STDERR @_;
-    if (!$self->{noexit}) {
-        _log(2,"Exiting with status ".REJECTED);
+    _log(2,"Exiting with status ".REJECTED);
 	exit REJECTED;
-    }
 }
 
 sub pipe {
 	my $self = shift;
 	my $file = shift;
 	return $self->{pipe}->() if exists $self->{pipe};
-        _log(1, "Piping to $file");
+    _log(1, "Piping to $file");
 	unless (open (PIPE, "|$file")) {
-            _log(0, "Couldn't open pipe $file: $!");
-            $self->accept();
-        }
+        _log(0, "Couldn't open pipe $file: $!")
+        $self->accept();
+    }
 	$self->{obj}->print(\*PIPE);
 	close PIPE;
-        _log(3,"Pipe closed with status $?");
-        if (!$self->{noexit}) {
-            _log(2,"Exiting with status ".DELIVERED);
-            exit DELIVERED;
-        }
+    _log(3,"Pipe closed with status $?");
+    _log(2,"Exiting with status ".DELIVERED);
+	exit DELIVERED;
 }
 
-sub header { $_[0]->{obj}->head->as_string() }
-sub put_header { $_[0]->{obj}->head->add($_[1],$_[2]); }
-sub replace_header { $_[0]->{obj}->head->replace ($_[1],$_[2]); }
-sub delete_header { $_[0]->{obj}->head->delete ($_[1]); }
-sub tidy { $_[0]->{obj}->tidy_body() }
+sub tidy { $_[0]->{obj}->tidy() }
 sub from { $_[0]->{obj}->head->get("From") }
 sub to { $_[0]->{obj}->head->get("To") }
 sub subject { $_[0]->{obj}->head->get("Subject") }
 sub bcc { $_[0]->{obj}->head->get("Bcc") }
 sub cc { $_[0]->{obj}->head->get("Cc") }
-sub body { $_[0]->{obj}->body }
-sub received { $_[0]->{obj}->head->get("Received") }
+sub received { $_[0]->{obj}->head->get("Recieved") }
 sub get { $_[0]->{obj}->head->get($_[1]) }
 sub resend {$_[0]->{obj}->smtpsend(To => $_[1]) }
-sub ignore { _log(1,"Ignoring"); exit DELIVERED unless $_[0]->{noexit} }
+sub ignore { _log(1,"Ignoring"); exit DELIVERED }
 
 sub myALRM { die "alarm\n" }
+sub rblcheck {
+my ($self, $timeout) = (shift, shift);
+_log(1,"Performing RBL check");
+my @recieved      = $self->received;
+my $rcvcount      = 0;
+$timeout = 10 unless defined $timeout;
+
+# Catch ALRM signals so we can timeout DNS lookups
+$SIG{ALRM} = 'myALRM';
+&myALRM() if 0;              # make -w shut up
+for (@recieved) {
+    my $x = _checkit($rcvcount,$_,$timeout);
+    if ($x) {
+        _log(2, "Check returned $x after ".(1+$rcvcount)." recieved headers");
+        return $x
+    }
+    $rcvcount++;  # Any further Received lines won't be the first.
+}
+_log(2, "Check was fine");
+return '';
+}
+
+sub checkit {
+    my $MAPS          = '.rbl.maps.vix.com';
+    my $OK            = '';
+    my $InvalidIP     = '1 Invalid IP address ';
+    my $RcvBlackHole  = '2 Received from RBL-registered spam site ';
+    my $RlyBlackHole  = '3 Relayed through RBL-registered spam site ';
+
+   my($relay,$rcvd,$timeout) = @_;
+   my($IP,@IP) = $rcvd =~ /\[((\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3}))\]/;
+   my($name,$x);
+   # We can't complain if there's no IP address in this Received header.
+   return ($OK) unless defined $IP;
+   # Outer limits lose
+   return ($InvalidIP.$IP) if $IP eq '0.0.0.0';
+   return ($InvalidIP.$IP) if $IP eq '255.255.255.255';
+   # All @IP components must be >= 0 and <= 255
+   foreach $x ( @IP ) {
+      return ($InvalidIP.$IP) if $x > 255;
+      return ($InvalidIP.$IP) if $x =~ /^0\d/;    # no leading zeroes allowed
+   }
+   #
+   # Wrap the gethostbyname call with eval in case it times out.
+   #
+   eval {
+      alarm($timeout);
+      ($name) = gethostbyname(join('.',reverse @IP) . $MAPS);
+      alarm(0);
+   };
+   return($OK) if $@ =~ /^alarm/;  # Timed out.  Let it through.
+   return($OK) unless $name;       # If it's ok with MAPS, it's OK with us.
+   return($relay ? $RlyBlackHole.$IP : $RcvBlackHole.$IP);
+}
+
 
 1;
 __END__
@@ -169,7 +192,7 @@ Mail::Audit - Library for creating easy mail filters
 
 =head1 SYNOPSIS
 
-	use Mail::Audit; # use Mail::Audit qw(...plugins...);
+	use Mail::Audit;
 	my $mail = Mail::Audit->new;
 	$mail->pipe("listgate p5p") if ($mail->from =~ /perl5-porters/);
 	$mail->accept("perl) if ($mail->from =~ /perl/);
@@ -193,21 +216,22 @@ to stick in a F<.forward> file or similar.
 
 =over 4
 
-=item C<new(%options)>
+=item C<new(%overrides)>
 
-The constructor reads a mail message from C<STDIN> (or, if the C<data>
-option is set, from an array reference) and creates a C<Mail::Audit>
-object from it, to be manipulated by the following methods.
+The constructor reads a mail message from C<STDIN> and creates a
+C<Mail::Audit> object from it, to be manipulated by the following
+methods.
 
-Other options include the C<accept>, C<reject> or C<pipe> keys, which
-specify subroutine references to override the methods with those names.
+You may optionally specify a hash with C<accept>, C<reject> or C<pipe>
+keys and with subroutine references to override the methods with those
+names. For example, people using MH as their mail handler will want to
+override C<accept> to reflect the local delivery method of that mailer.
 
-You may also specify C<< log => $logfile >> to write a debugging log; you
+You may also specify C<log => $logfile> to write a debugging log; you
 can set the verbosity of the log with the C<loglevel> key, on a scale of
 1 to 4. If you specify a log level without a log file, logging will be
 written to F</tmp/you-audit.log> where F<you> is replaced by your user
-name. If you specify C<< noexit => 1 >>, C<Mail::Audit> will not exit
-after delivering the mail, but continue running your filter. 
+name.
 
 =back
 
@@ -238,6 +262,12 @@ bounce.
 This merely ignores the email, dropping it into the bit bucket for
 eternity.
 
+=item C<rblcheck([$timeout])>
+
+Attempts to check the mail headers with the Relay Blackhole List. 
+Returns false if the headers check out fine or the query times out,
+returns a reason if the mail is considered spam.
+
 =item C<pipe($program)>
 
 This opens a pipe to an external program and feeds the mail to it.
@@ -250,27 +280,7 @@ Tidies up the email as per L<Mail::Internet>
 
 Retrieves the named header from the mail message.
 
-=item C<put_header($header, $value)>
-
-Inserts a new header into the mail message with the given value.
-
-=item C<replace_header($header, $value)>
-
-Removes the old header, adds a new one.
-
-=item C<delete_header($header)>
-
-Guess.
-
-=item C<body>
-
-Returns a reference to an array of lines in the body of the email.
-
-=item C<header>
-
-Returns the header as a single string.
-
-=item C<resend($address)>
+=item C<resent($address)>
 
 Bounces the email in its entirety to another address.
 
@@ -304,14 +314,10 @@ bcc
 
 =back
 
-=head1 LICENSE
-
-The usual. This program is free software; you can redistribute it
-and/or modify it under the same terms as Perl itself.
-
 =head1 BUGS
 
-Fewer than before.
+Only tested on qmail and postfix, and I don't know how universally the
+exit code 100 means reject.
 
 =head1 AUTHOR
 
